﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


/** This script represents a generic menu manager. It is mainly aimed at 
representing the navigation through menus, but it can also be used
to navigate through screens (even if there is not any menu behind). 
The different screens register on this manager. This manager can navigate
by disabling all registered elements and enabling one particular item.
 There must be only one instance of this class so that there are not
 conflicts in the udpates. If several menu elements share some objects,
 they can have some parent common item to maintin this elements activated
 in all these. For now, this is only implemented up to one level. */
public class MenuManager : MonoBehaviour {
	// It indicates the initial menu item to start, from the editor
	public string initialMenuElement;
	/* The items of the menu that must be set in the editor in the manager */ 
	public MenuElement[] items;
	// It indicates the current menu
	private MenuElement currentMenuItem;
	// Single instance. This only makes sense if there is one menu manager
	private static MenuManager instance;

	// Use this with caution. It does not suply the register. It was only added
	// in some cases that the instance was needed (e.g. for using modal windows
	// for elements from the ones registered in the menu manager)
	public static MenuManager Instance(){
		if (instance == null)
			instance = FindObjectOfType<MenuManager> ();
		return instance;
	}

	/** It registers the game object of a menu item in the list of items */
	/*public void register (MenuElement item){
		items.AddLast (item);
	}*/
	/** It navigates to certain screen. Basically it enables the object
	 * with the corresponding name and its parent if different from null.
	 * It disables all the other menu elements. */
	public void GoTo(string name){
		// Find the menu element
		bool found = false;
		foreach (MenuElement item in items) {
			if (item.gameObject.name.Equals (name)) {
				ActivateMenuElement(item);
				found = true;
				currentMenuItem = item;
			}else {
				item.gameObject.SetActive(false);
			}
		}
		// Find the parent and activate
		if(found && (currentMenuItem.parentMenuElement!=null))
			ActivateMenuElement(currentMenuItem.parentMenuElement);
		// Report possible error
		if (!found && !name.Equals(""))
			Debug.Log ("Error while going to the menu item " + name);
	}

	// Find an element on the list
	private MenuElement FindMenuElement(string name){
		foreach (MenuElement item in items) {
			if (item.gameObject.name.Equals (name)) {
				return item;
			}
		}
		Debug.Log ("Error, not found element " + name);
		return null;
	}



	// It activates a menu element
	private void ActivateMenuElement(MenuElement item){
		item.enabled=true;
		item.gameObject.SetActive(true);
	}
	// It goes back to the previous menu item; or the app quits if there is not any 
	// preceeding item defined and  the initial menu is active
	// The check of the initial menu allows that in some parts of the application
	// the Escape button is managed without quiting because this class
	// An element can be intentionally be set to itself to do nothing
	// until it is manually taken out the the SilentGoTo
	public void GoBack(){
		MenuElement precedingItem = currentMenuItem.precedes;
		if (precedingItem == null) {
				Application.Quit ();
			//}
		} else {
			// Only changes if it actually is different. 
			if(!(precedingItem.gameObject.name.Equals(currentMenuItem.gameObject.name)))
				GoTo (precedingItem.gameObject.name);
		}
	}
	// At the beginning, it goes to the initial menu item
	void Start (){
		//initialGameObject = FindMenuElement (initialMenuElement).gameObject;
		GoTo (initialMenuElement);
	}
	// When pressing the back/escape button, it goes to the menu item that precedes
	// the current one
	void Update(){
		if (Input.GetKeyDown (KeyCode.Escape)) {
			GoBack();
		}
	}

}

