﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

// It handles a starplot, and is attached to the starplot world  object, 
// where the lines will be added. It uses several constants from the 
// Visual constants class  
public class StarplotHandler : MonoBehaviour {
    // Reference to the prefab of the line
    public GameObject starplotLinePrefab;
    // Reference to the prefab of the line
    public GameObject starplotTrianglePrefab;
    // Reference to the prefab of label with the image and text
    public GameObject starplotLabelImageTextPrefab;
	// Reference to the container of lines and edges
    public GameObject starplotContainer;
    // Reference to a canvas for containing labels
    public GameObject labelsContainer;
    // Center of the starplot in the world space
	public Vector2 center;  //  = new Vector2(0, 1.5f)
	// Radious of the starplot in world space units (normally for a othogonic
	// camera of size 5).
	private float radius = 1.5f;
	// Ratio for labels: a little more than 1, so it is painted
	// outside the starplot
	private float ratioRadiusLabel =1.6f;

	// Creates the labels from a list of Sprite 2D (i.e images) and texts
	public void CreateLabelsFromImagesTexts(Sprite[] sprites, string[] texts)
	{
		float angle = 0;
		float incAngle = 2 * Mathf.PI / ((float)sprites.Length);
		for (int i = 0; i < sprites.Length; i++)
		{
			Vector2 pos = CreatePositionStarEnd(angle,ratioRadiusLabel);
			Vector3 screenPos = Camera.main.WorldToScreenPoint(pos);
			GameObject label = Instantiate(starplotLabelImageTextPrefab);
			label.transform.SetParent(labelsContainer.transform,false);
			label.transform.position = screenPos;
			Image imageLabel = label.GetComponentInChildren<Image>();
			imageLabel.overrideSprite = sprites[i];
			label.GetComponentInChildren<Text> ().text = texts [i];
			angle += incAngle;
		}
	}
     
    // It creates the starplot from a list of values, considering
    // that the maximum values reached and end of the starplot 
    public void CreateStarplotFromValues (float[] values)
    {
        // Divide all the values by the maximum value
        float maxValue = Mathf.Max(values);
        float[] ratios = new float[values.Length];
        if (maxValue > 0)
        {
            for(int i = 0; i < values.Length; i++)
            {
                ratios[i] = values[i] / maxValue;
            }
                
        }
        // Create it from ratios
        CreateStarplotFromRatios(ratios);
    }

    // It creates a starplot from certain ratio values in the interval
    // [0,1] so that the 1 reaches the end of the starplot
    public void CreateStarplotFromRatios(float[] ratios)
    {
        // Remove all previous elements
        RemoveAllElements();
        // Draw the edges and radii
        DrawEdgesStarplot(ratios);
        DrawRadii(ratios.Length);
    }
    // Remove all the elements (radii and edges)
    public void RemoveAllElements()
    {
        for (int i = starplotContainer.transform.childCount - 1; i >= 0; i--)
        {
            Transform child = starplotContainer.transform.GetChild(i);
            Destroy(child.gameObject);
        }
		for (int i = labelsContainer.transform.childCount - 1; i >= 0; i--)
		{
			Transform child = labelsContainer.transform.GetChild(i);
			Destroy(child.gameObject);
		}
    }

    /* Creates  a starplot from a list of real values. that represent ratios
     * in the interval [0, 1]*/
    private void DrawEdgesStarplot(float[] ratios)
    {
        float angle = 0;
        float incAngle = 2 * Mathf.PI / ((float) ratios.Length);
        // It assigs the first point of the star as the previous one
        Vector2 firstPosValue = CreatePositionStarEnd(angle, ratios[0]);
        Vector2 prevPosValue = firstPosValue;
        Vector2 posValue;
        for (int i = 1; i< ratios.Length; i++)
        {
            // It generates each edge of of the star
            angle += incAngle;
            posValue = CreatePositionStarEnd(angle, ratios[i]);
            DrawTriangle(prevPosValue, posValue);
            DrawLine(prevPosValue, posValue,VisualConstants.starplotEdgeWidth);
            prevPosValue = posValue;
        }
        // The last point of the star connects to the first one
        DrawTriangle(prevPosValue, firstPosValue);
        DrawLine(prevPosValue, firstPosValue,VisualConstants.starplotEdgeWidth);
    }

    /* It determines the postion of the star for a given angle, and a given
     * ratio value that represents the intensity of certain value in the
     * [0,1 interval]. It considers the center and radius. 
     Exceptionally, it can be used with a ratio a little more than one
     to place a label. */
    private Vector2 CreatePositionStarEnd(float angle, float ratio)
    {
        Vector2 direction = new Vector2(Mathf.Cos(angle), Mathf.Sin(angle));
        Vector2 maxDirection = radius * direction;
        Vector2 ratedDirection = ratio * maxDirection;
        Vector2 positionStar = center + ratedDirection;
        return positionStar;
    }
    // Create the end of the radio for a given angle
    private Vector2 CreateEndRadius (float angle)
    {
        return CreatePositionStarEnd(angle, 1);
    }
    // Creates every radius of the starplot 
    // It recieves the number of radii (Radii is the plural of radius)
    private void DrawRadii(int numRadii)
    {
        float angle = 0;
        float incAngle = 2 * Mathf.PI / ((float) numRadii);
        for (int i = 0; i < numRadii; i++)
        {
            DrawLine(center, CreateEndRadius(angle), VisualConstants.starplotRadiusWidth);
            angle += incAngle;
        }
    }
    /* It draws a line from one point to another */
    private void DrawLine(Vector2 init, Vector2 end, float lineWidth)
    {
        GameObject line = Object.Instantiate(starplotLinePrefab);
        line.GetComponent<LineHandler>().PlaceLine(init, end, lineWidth);
        line.transform.parent = starplotContainer.transform;
    }
    /* It draws a triangle for two star ends */
    private void DrawTriangle(Vector2 init, Vector2 end)
    {
        GameObject line = Object.Instantiate(starplotTrianglePrefab);
        line.GetComponent<LineHandler>().PlaceTriangle(center,init, end);
        line.transform.parent = starplotContainer.transform;
    }
}





