﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using System;

/* This class represents the main user interface (UI) of the simulator, connecting 
 * the UI with the simulator. 
*/
public class MainGUI : MonoBehaviour {
	// Reference to the simulation
	Simulation sim;
	// Agents that need to be accessed at the end of the simulation
	ObserverAgent observerAgent;
	ManagerAgent managerAgent;
	// Texts with the inputs
	public Text honestUsersText;
	public Text selfishUsersText;
	public Text numServicesText;
	public Text numIterationsText;
	public Dropdown strategyDropdown;
	// Suffix strategy for reflection
	string suffixStrategy = "PriorityAgent";

	// It runs the simulation when invoked by the user from the UI
	public void RunSim(){
		// Set the params of the imulation
		SetParams();
		// Perform several simulations for the experiments
		for (int i = 0; i < Experimentation.numSims; i++) {
			// Create the simulations and the corresonding agents
			sim = new Simulation ();
			CreateAgents ();
			// Get the number of iterations and run the simulation
			int numIterations = (int) GetNumberFromText(numIterationsText);
			sim.RunSim (numIterations);
			// Obtain the results and save these
			float[] resultsObserver = observerAgent.CalculateResultsMetrics ();
			List<float> results = new List<float> ();
			foreach (float value in resultsObserver)
				results.Add (value);
			Experimentation.SaveResults (results,Experimentation.pathResults);
		}
		Debug.Log("The results have been saved in "+Experimentation.pathResults);
		// Presents the last results, and indicate where the results have been saved
		ResultsPresenter.Instance().ShowEvolutions(observerAgent);
		ResultsPresenter.Instance().ShowStarplotResults(observerAgent);
		// It saves the evolution of the last simulation, it the experimentation flag is on
		observerAgent.SaveEvolutionIntoFile();
	}
	// It sets the params of the simulation with the data provided by the user
	private void SetParams(){
		Params.numServices = (int)GetNumberFromText (numServicesText);
	}
	// It creates all the agents of the simulation
	private void CreateAgents(){
		// Cretate the manager
		managerAgent = new ManagerAgent(sim);
		// Create a pairs of user-priority agents, using honest agents, and then selfish
		int numUsers = (int) GetNumberFromText(honestUsersText);
		for (int i = 0; i < numUsers; i++) 
			Connect (new StochasticUserAgent (sim), managerAgent);
		numUsers = (int) GetNumberFromText(selfishUsersText);
		for (int i = 0; i < numUsers; i++) 
			Connect (new SelfishUserAgent (sim), managerAgent);		
		// Create the observer agent (in this case a type of user)
		observerAgent = new ObserverAgent(sim);
		Connect (observerAgent, managerAgent);
		observerAgent.AddInitialValues ();
		// Adds the manager at the end so it can attend requests in the same iteration
		sim.agents.Add (managerAgent);
	}
	// It connects a user agent to the manager agent through a new priority agent, and
	// adds it to the simulation
	private void Connect(UserAgent userAgent, ManagerAgent managerAgent){
		PriorityAgent priorityAgent = CreateStrategyAgent();
		userAgent.SetPriorityAgent (priorityAgent);
		sim.agents.Add (userAgent);
	}
	// Create the strategy from the dropdown list. If no-one is
	// selected it uses the default behavior
	private PriorityAgent CreateStrategyAgent(){
		PriorityAgent priorityAgent;
		// Get the selected strategy from the dropdown list
		int selectedIndex = strategyDropdown.value;
		string nameStrategy = strategyDropdown.options [selectedIndex].text;
		nameStrategy += suffixStrategy;
		Type typeStrategy = Type.GetType (nameStrategy);
		if (typeStrategy == null) {
			Debug.Log ("Error while loading the strategy " + nameStrategy);
			priorityAgent = new PriorityAgent (sim,managerAgent);
		} else {		
			System.Object[] paramsStrategy = new System.Object[2];
			paramsStrategy [0] = sim;
			paramsStrategy [1] = managerAgent;
			priorityAgent = Activator.CreateInstance (typeStrategy, paramsStrategy)
				as PriorityAgent;
		}
		sim.agents.Add (priorityAgent);
		return priorityAgent;
	}
	// It gets an integer from a text, and shows a message in case of error
	public float GetNumberFromText(Text text){
		float result;
		if (float.TryParse (text.text, out result))
			return result;
		else {
			Debug.Log ("Error in the format of a number.");
			return 0;
		}
	}
}
	